@@ -1,4 +1,4 @@
-# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/lib/Data/Localize/Gettext.pm 100692 2009-02-15T05:17:58.132409Z daisuke  $
+# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/lib/Data/Localize/Gettext.pm 100948 2009-02-20T04:07:52.965534Z daisuke  $
 
 package Data::Localize::Gettext;
 use utf8;
@@ -99,7 +99,7 @@ my @fuzzy;
             $lexicon{ $var{msgid} } = '';
         }
         if ( $var{msgid} eq '' ) {
-            push @metadata, parse_metadata( $var{msgstr} );
+            push @metadata, $self->parse_metadata( $var{msgstr} );
         }
         else {
             push @comments, $var{msgid}, $var{msgcomment};
@@ -171,6 +171,24 @@ sub lexicon_merge {
     }
 }
 
+sub parse_metadata {
+    my $self = shift;
+    return map {
+              (/^([^\x00-\x1f\x80-\xff :=]+):\s*(.*)$/)
+            ? ( $1 eq 'Content-Type' )
+                ? do {
+                    my $enc = $2;
+                    if ( $enc =~ /\bcharset=\s*([-\w]+)/i ) {
+                        $self->encoding($1);
+                    }
+                    ( "__Content-Type", $enc );
+                }
+                : ( "__$1", $2 )
+            : ();
+    } split( /\r*\n+\r*/, $_[0]);
+}
+
+
 1;
 
 __END__
@@ -203,4 +221,37 @@ Registeres this localizer
 
 Currently, strings are assumed to be utf-8,
 
+=head1 AUTHOR
+
+Daisuke Maki C<< <daisuke@endeworks.jp> >>
+
+Parts of this code stolen from Locale::Maketext::Lexicon::Gettext.
+
+=head1 COPYRIGHT
+
+=over 4
+
+=item The "MIT" License
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the
+"Software"), to deal in the Software without restriction, including
+without limitation the rights to use, copy, modify, merge, publish,
+distribute, sublicense, and/or sell copies of the Software, and to
+permit persons to whom the Software is furnished to do so, subject to
+the following conditions:
+
+The above copyright notice and this permission notice shall be included
+in all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+=back
+
 =cut
\ No newline at end of file
